@@ -1,10 +1,59 @@
 // SPDX-License-Identifier: MIT
 // Copyright (C) 2018-present iced project and contributors
 
+macro_rules! write_op0_reg {
+	($instruction:ident, $expr:expr) => {
+		const_assert_eq!(OpKind::Register as u32, 0);
+		//instruction.set_op0_kind(OpKind::Register);
+		debug_assert!($expr < IcedConstants::REGISTER_ENUM_COUNT as u32);
+		$instruction.set_op0_register(unsafe { mem::transmute($expr as RegisterUnderlyingType) });
+	};
+}
+
+macro_rules! write_op1_reg {
+	($instruction:ident, $expr:expr) => {
+		const_assert_eq!(OpKind::Register as u32, 0);
+		//instruction.set_op1_kind(OpKind::Register);
+		debug_assert!($expr < IcedConstants::REGISTER_ENUM_COUNT as u32);
+		$instruction.set_op1_register(unsafe { mem::transmute($expr as RegisterUnderlyingType) });
+	};
+}
+
+macro_rules! write_op2_reg {
+	($instruction:ident, $expr:expr) => {
+		const_assert_eq!(OpKind::Register as u32, 0);
+		//instruction.set_op2_kind(OpKind::Register);
+		debug_assert!($expr < IcedConstants::REGISTER_ENUM_COUNT as u32);
+		$instruction.set_op2_register(unsafe { mem::transmute($expr as RegisterUnderlyingType) });
+	};
+}
+
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop")))]
+macro_rules! write_op3_reg {
+	($instruction:ident, $expr:expr) => {
+		const_assert_eq!(OpKind::Register as u32, 0);
+		//instruction.set_op3_kind(OpKind::Register);
+		debug_assert!($expr < IcedConstants::REGISTER_ENUM_COUNT as u32);
+		$instruction.set_op3_register(unsafe { mem::transmute($expr as RegisterUnderlyingType) });
+	};
+}
+
+pub(super) mod d3now;
+#[cfg(not(feature = "no_evex"))]
+pub(super) mod evex;
+pub(super) mod fpu;
+pub(super) mod legacy;
+#[cfg(feature = "mvex")]
+pub(super) mod mvex;
+pub(super) mod tables;
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop")))]
+pub(super) mod vex;
+
 use crate::decoder::*;
 use crate::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
+use core::convert::TryInto;
 
 // SAFETY:
 //	code: let this = unsafe { &*(self_ptr as *const Self) };
@@ -25,7 +74,7 @@ pub(super) fn is_null_instance_handler(handler: &OpCodeHandler) -> bool {
 #[inline]
 pub(super) fn get_null_handler() -> (OpCodeHandlerDecodeFn, &'static OpCodeHandler) {
 	// SAFETY: it's #[repr(C)] and the first part is an `OpCodeHandler`
-	(NULL_HANDLER.decode, unsafe { &*(&NULL_HANDLER as *const _ as *const OpCodeHandler) })
+	(OpCodeHandler_Invalid::decode, unsafe { &*(&NULL_HANDLER as *const _ as *const OpCodeHandler) })
 }
 
 #[allow(trivial_casts)]
@@ -33,7 +82,7 @@ pub(super) fn get_null_handler() -> (OpCodeHandlerDecodeFn, &'static OpCodeHandl
 #[inline]
 pub(super) fn get_invalid_handler() -> (OpCodeHandlerDecodeFn, &'static OpCodeHandler) {
 	// SAFETY: it's #[repr(C)] and the first part is an `OpCodeHandler`
-	(INVALID_HANDLER.decode, unsafe { &*(&INVALID_HANDLER as *const _ as *const OpCodeHandler) })
+	(OpCodeHandler_Invalid::decode, unsafe { &*(&INVALID_HANDLER as *const _ as *const OpCodeHandler) })
 }
 
 #[allow(trivial_casts)]
@@ -41,36 +90,31 @@ pub(super) fn get_invalid_handler() -> (OpCodeHandlerDecodeFn, &'static OpCodeHa
 #[inline]
 pub(super) fn get_invalid_no_modrm_handler() -> (OpCodeHandlerDecodeFn, &'static OpCodeHandler) {
 	// SAFETY: it's #[repr(C)] and the first part is an `OpCodeHandler`
-	(INVALID_NO_MODRM_HANDLER.decode, unsafe { &*(&INVALID_NO_MODRM_HANDLER as *const _ as *const OpCodeHandler) })
+	(OpCodeHandler_Invalid::decode, unsafe { &*(&INVALID_NO_MODRM_HANDLER as *const _ as *const OpCodeHandler) })
 }
 
 #[rustfmt::skip]
 pub(super) static NULL_HANDLER: OpCodeHandler_Invalid = OpCodeHandler_Invalid {
-	decode: OpCodeHandler_Invalid::decode,
 	has_modrm: true,
 };
 #[rustfmt::skip]
 pub(super) static INVALID_HANDLER: OpCodeHandler_Invalid = OpCodeHandler_Invalid {
-	decode: OpCodeHandler_Invalid::decode,
 	has_modrm: true,
 };
 #[rustfmt::skip]
 pub(super) static INVALID_NO_MODRM_HANDLER: OpCodeHandler_Invalid = OpCodeHandler_Invalid {
-	decode: OpCodeHandler_Invalid::decode,
 	has_modrm: false,
 };
 
 #[repr(C)]
 pub(super) struct OpCodeHandler {
 	pub(super) has_modrm: bool,
-	pub(super) decode: OpCodeHandlerDecodeFn,
 }
 
 #[allow(non_camel_case_types)]
 #[repr(C)]
 pub(super) struct OpCodeHandler_Invalid {
 	has_modrm: bool,
-	pub(super) decode: OpCodeHandlerDecodeFn,
 }
 
 impl OpCodeHandler_Invalid {
@@ -83,17 +127,18 @@ impl OpCodeHandler_Invalid {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Simple {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	code: Code,
 }
 
 impl OpCodeHandler_Simple {
-	pub(super) fn new(code: Code) -> Self {
-		Self { has_modrm: false, decode: OpCodeHandler_Simple::decode, code }
+	#[inline]
+	pub(super) fn new(code: Code) -> (OpCodeHandlerDecodeFn, Self) {
+		(OpCodeHandler_Simple::decode, Self { has_modrm: false, code })
 	}
 
-	pub(super) fn new_modrm(code: Code) -> Self {
-		Self { has_modrm: true, decode: OpCodeHandler_Simple::decode, code }
+	#[inline]
+	pub(super) fn new_modrm(code: Code) -> (OpCodeHandlerDecodeFn, Self) {
+		(OpCodeHandler_Simple::decode, Self { has_modrm: true, code })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, _decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -102,32 +147,50 @@ impl OpCodeHandler_Simple {
 	}
 }
 
+#[allow(non_camel_case_types)]
+#[repr(C)]
+pub(super) struct OpCodeHandler_Int3 {
+	has_modrm: bool,
+}
+
+impl OpCodeHandler_Int3 {
+	#[inline]
+	pub(super) fn new() -> (OpCodeHandlerDecodeFn, Self) {
+		(OpCodeHandler_Int3::decode, Self { has_modrm: false })
+	}
+
+	fn decode(_self_ptr: *const OpCodeHandler, _decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
+		instruction.set_code(Code::Int3);
+	}
+}
+
 #[allow(non_camel_case_types)]
 #[repr(C)]
 pub(super) struct OpCodeHandler_Group8x8 {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
-	table_low: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
-	table_high: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
+	table_low: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 8]>,
+	table_high: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 8]>,
 }
 
 impl OpCodeHandler_Group8x8 {
+	#[inline]
+	#[allow(clippy::unwrap_used)]
 	pub(super) fn new(
 		table_low: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>, table_high: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
-	) -> Self {
-		debug_assert_eq!(table_low.len(), 8);
-		debug_assert_eq!(table_high.len(), 8);
-		Self { has_modrm: true, decode: OpCodeHandler_Group8x8::decode, table_low, table_high }
+	) -> (OpCodeHandlerDecodeFn, Self) {
+		let table_low = table_low.into_boxed_slice().try_into().ok().unwrap();
+		let table_high = table_high.into_boxed_slice().try_into().ok().unwrap();
+		(OpCodeHandler_Group8x8::decode, Self { has_modrm: true, table_low, table_high })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
 		debug_assert!(decoder.state.reg <= 7);
 		let (decode, handler) = if decoder.state.mod_ == 3 {
-			// SAFETY: reg <= 7 and table_high.len() == 8 (see ctor)
+			// SAFETY: reg <= 7 and table_high.len() == 8 (see field type)
 			unsafe { *this.table_high.get_unchecked(decoder.state.reg as usize) }
 		} else {
-			// SAFETY: reg <= 7 and table_low.len() == 8 (see ctor)
+			// SAFETY: reg <= 7 and table_low.len() == 8 (see field type)
 			unsafe { *this.table_low.get_unchecked(decoder.state.reg as usize) }
 		};
 		(decode)(handler, decoder, instruction);
@@ -138,36 +201,37 @@ impl OpCodeHandler_Group8x8 {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Group8x64 {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
-	table_low: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
-	table_high: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
+	table_low: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 8]>,
+	table_high: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 0x40]>,
 }
 
 impl OpCodeHandler_Group8x64 {
+	#[inline]
+	#[allow(clippy::unwrap_used)]
 	pub(super) fn new(
 		table_low: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>, table_high: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
-	) -> Self {
-		debug_assert_eq!(table_low.len(), 8);
-		debug_assert_eq!(table_high.len(), 0x40);
-		Self { has_modrm: true, decode: OpCodeHandler_Group8x64::decode, table_low, table_high }
+	) -> (OpCodeHandlerDecodeFn, Self) {
+		let table_low = table_low.into_boxed_slice().try_into().ok().unwrap();
+		let table_high = table_high.into_boxed_slice().try_into().ok().unwrap();
+		(OpCodeHandler_Group8x64::decode, Self { has_modrm: true, table_low, table_high })
 	}
 
 	#[allow(trivial_casts)]
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
 		let (decode, handler) = if decoder.state.mod_ == 3 {
-			// SAFETY: table_high.len() == 0x40 (see ctor) and index <= 0x3F due to masking `modrm`
+			// SAFETY: table_high.len() == 0x40 (see field type) and index <= 0x3F due to masking `modrm`
 			let (decode, handler) = unsafe { *this.table_high.get_unchecked((decoder.state.modrm & 0x3F) as usize) };
 			if handler as *const _ as *const u8 == &NULL_HANDLER as *const _ as *const u8 {
 				debug_assert!(decoder.state.reg <= 7);
-				// SAFETY: reg <= 7 and table_low.len() == 8 (see ctor)
+				// SAFETY: reg <= 7 and table_low.len() == 8 (see field type)
 				unsafe { *this.table_low.get_unchecked(decoder.state.reg as usize) }
 			} else {
 				(decode, handler)
 			}
 		} else {
 			debug_assert!(decoder.state.reg <= 7);
-			// SAFETY: reg <= 7 and table_low.len() == 8 (see ctor)
+			// SAFETY: reg <= 7 and table_low.len() == 8 (see field type)
 			unsafe { *this.table_low.get_unchecked(decoder.state.reg as usize) }
 		};
 		(decode)(handler, decoder, instruction);
@@ -178,20 +242,21 @@ impl OpCodeHandler_Group8x64 {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Group {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
-	group_handlers: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>,
+	group_handlers: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 8]>,
 }
 
 impl OpCodeHandler_Group {
-	pub(super) fn new(group_handlers: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>) -> Self {
-		debug_assert_eq!(group_handlers.len(), 8);
-		Self { has_modrm: true, decode: OpCodeHandler_Group::decode, group_handlers }
+	#[inline]
+	#[allow(clippy::unwrap_used)]
+	pub(super) fn new(group_handlers: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>) -> (OpCodeHandlerDecodeFn, Self) {
+		let group_handlers = group_handlers.into_boxed_slice().try_into().ok().unwrap();
+		(OpCodeHandler_Group::decode, Self { has_modrm: true, group_handlers })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
 		debug_assert!(decoder.state.reg <= 7);
-		// SAFETY: group_handlers.len() == 8 (see ctor) and reg <= 7
+		// SAFETY: group_handlers.len() == 8 (see field type) and reg <= 7
 		let (decode, handler) = unsafe { *this.group_handlers.get_unchecked(decoder.state.reg as usize) };
 		(decode)(handler, decoder, instruction);
 	}
@@ -201,41 +266,53 @@ impl OpCodeHandler_Group {
 #[repr(C)]
 pub(super) struct OpCodeHandler_AnotherTable {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	handlers: Box<[(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 0x100]>,
 }
 
 impl OpCodeHandler_AnotherTable {
 	#[allow(clippy::unwrap_used)]
-	pub(super) fn new(handlers: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>) -> Self {
-		let handlers = handlers.into_boxed_slice();
-		debug_assert_eq!(handlers.len(), 0x100);
-		// SAFETY: Size is verified above
-		let handlers = unsafe { Box::from_raw(Box::into_raw(handlers) as *mut [_; 0x100]) };
-		Self { has_modrm: false, decode: OpCodeHandler_AnotherTable::decode, handlers }
+	#[inline]
+	pub(super) fn new(handlers: Vec<(OpCodeHandlerDecodeFn, &'static OpCodeHandler)>) -> (OpCodeHandlerDecodeFn, Self) {
+		let handlers = handlers.into_boxed_slice().try_into().ok().unwrap();
+		(OpCodeHandler_AnotherTable::decode, Self { has_modrm: false, handlers })
 	}
 
+	#[allow(clippy::never_loop)]
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
-		decoder.decode_table(&this.handlers, instruction);
+		loop {
+			let b = read_u8_break!(decoder);
+			let (decode, handler) = this.handlers[b];
+			if handler.has_modrm {
+				let m = read_u8_break!(decoder) as u32;
+				decoder.state.modrm = m;
+				decoder.state.reg = (m >> 3) & 7;
+				decoder.state.mod_ = m >> 6;
+				decoder.state.rm = m & 7;
+				decoder.state.mem_index = (decoder.state.mod_ << 3) | decoder.state.rm;
+			}
+			(decode)(handler, decoder, instruction);
+			return;
+		}
+		decoder.state.flags |= StateFlags::IS_INVALID | StateFlags::NO_MORE_BYTES;
 	}
 }
 
-#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex")))]
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex"), feature = "mvex"))]
 #[allow(non_camel_case_types)]
 #[repr(C)]
 pub(super) struct OpCodeHandler_MandatoryPrefix2 {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	handlers: [(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 4],
 }
 
-#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex")))]
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex"), feature = "mvex"))]
 impl OpCodeHandler_MandatoryPrefix2 {
+	#[inline]
 	pub(super) fn new(
 		has_modrm: bool, handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler_66: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 		handler_f3: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler_f2: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		const_assert_eq!(DecoderMandatoryPrefix::PNP as u32, 0);
 		const_assert_eq!(DecoderMandatoryPrefix::P66 as u32, 1);
 		const_assert_eq!(DecoderMandatoryPrefix::PF3 as u32, 2);
@@ -249,50 +326,50 @@ impl OpCodeHandler_MandatoryPrefix2 {
 		debug_assert_eq!(handlers[1].1.has_modrm, has_modrm);
 		debug_assert_eq!(handlers[2].1.has_modrm, has_modrm);
 		debug_assert_eq!(handlers[3].1.has_modrm, has_modrm);
-		Self { decode: OpCodeHandler_MandatoryPrefix2::decode, has_modrm, handlers }
+		(OpCodeHandler_MandatoryPrefix2::decode, Self { has_modrm, handlers })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
 		debug_assert!(
-			decoder.state.encoding() == EncodingKind::VEX
-				|| decoder.state.encoding() == EncodingKind::EVEX
-				|| decoder.state.encoding() == EncodingKind::XOP
+			decoder.state.encoding() == EncodingKind::VEX as u32
+				|| decoder.state.encoding() == EncodingKind::EVEX as u32
+				|| decoder.state.encoding() == EncodingKind::XOP as u32
+				|| decoder.state.encoding() == EncodingKind::MVEX as u32
 		);
-		let (decode, handler) = unsafe { *this.handlers.get_unchecked(decoder.state.mandatory_prefix as usize) };
+		let (decode, handler) = this.handlers[decoder.state.mandatory_prefix as usize];
 		(decode)(handler, decoder, instruction);
 	}
 }
 
-#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex")))]
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex"), feature = "mvex"))]
 #[allow(non_camel_case_types)]
 #[repr(C)]
 pub(super) struct OpCodeHandler_W {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	handlers: [(OpCodeHandlerDecodeFn, &'static OpCodeHandler); 2],
 }
 
-#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex")))]
+#[cfg(any(not(feature = "no_vex"), not(feature = "no_xop"), not(feature = "no_evex"), feature = "mvex"))]
 impl OpCodeHandler_W {
+	#[inline]
 	pub(super) fn new(
 		handler_w0: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler_w1: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(handler_w0.1));
 		debug_assert!(!is_null_instance_handler(handler_w1.1));
-		Self { decode: OpCodeHandler_W::decode, has_modrm: true, handlers: [handler_w0, handler_w1] }
+		(OpCodeHandler_W::decode, Self { has_modrm: true, handlers: [handler_w0, handler_w1] })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
 		let this = unsafe { &*(self_ptr as *const Self) };
 		debug_assert!(
-			decoder.state.encoding() == EncodingKind::VEX
-				|| decoder.state.encoding() == EncodingKind::EVEX
-				|| decoder.state.encoding() == EncodingKind::XOP
+			decoder.state.encoding() == EncodingKind::VEX as u32
+				|| decoder.state.encoding() == EncodingKind::EVEX as u32
+				|| decoder.state.encoding() == EncodingKind::XOP as u32
+				|| decoder.state.encoding() == EncodingKind::MVEX as u32
 		);
-		const_assert_eq!(StateFlags::W, 0x80);
-		let index = (decoder.state.flags >> 7) & 1;
-		let (decode, handler) = unsafe { *this.handlers.get_unchecked(index as usize) };
+		let (decode, handler) = this.handlers[((decoder.state.flags & StateFlags::W) != 0) as usize];
 		(decode)(handler, decoder, instruction);
 	}
 }
@@ -301,18 +378,18 @@ impl OpCodeHandler_W {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Bitness {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	handler1632: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	handler64: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 }
 
 impl OpCodeHandler_Bitness {
+	#[inline]
 	pub(super) fn new(
 		handler1632: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler64: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(handler1632.1));
 		debug_assert!(!is_null_instance_handler(handler64.1));
-		Self { decode: OpCodeHandler_Bitness::decode, has_modrm: false, handler1632, handler64 }
+		(OpCodeHandler_Bitness::decode, Self { has_modrm: false, handler1632, handler64 })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -329,18 +406,18 @@ impl OpCodeHandler_Bitness {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Bitness_DontReadModRM {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	handler1632: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	handler64: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 }
 
 impl OpCodeHandler_Bitness_DontReadModRM {
+	#[inline]
 	pub(super) fn new(
 		handler1632: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler64: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(handler1632.1));
 		debug_assert!(!is_null_instance_handler(handler64.1));
-		Self { decode: OpCodeHandler_Bitness_DontReadModRM::decode, has_modrm: true, handler1632, handler64 }
+		(OpCodeHandler_Bitness_DontReadModRM::decode, Self { has_modrm: true, handler1632, handler64 })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -354,16 +431,18 @@ impl OpCodeHandler_Bitness_DontReadModRM {
 #[repr(C)]
 pub(super) struct OpCodeHandler_RM {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	reg: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	mem: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 }
 
 impl OpCodeHandler_RM {
-	pub(super) fn new(reg: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), mem: (OpCodeHandlerDecodeFn, &'static OpCodeHandler)) -> Self {
+	#[inline]
+	pub(super) fn new(
+		reg: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), mem: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(reg.1));
 		debug_assert!(!is_null_instance_handler(mem.1));
-		Self { has_modrm: true, decode: OpCodeHandler_RM::decode, reg, mem }
+		(OpCodeHandler_RM::decode, Self { has_modrm: true, reg, mem })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -377,7 +456,6 @@ impl OpCodeHandler_RM {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Options1632 {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	infos: [(OpCodeHandlerDecodeFn, &'static OpCodeHandler, u32); 2],
 	info_options: u32,
@@ -385,34 +463,40 @@ pub(super) struct OpCodeHandler_Options1632 {
 
 impl OpCodeHandler_Options1632 {
 	#[allow(trivial_casts)]
+	#[inline]
 	pub(super) fn new(
 		default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler1: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options1: u32,
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(default_handler.1));
 		debug_assert!(!is_null_instance_handler(handler1.1));
-		Self {
-			decode: OpCodeHandler_Options1632::decode,
-			has_modrm: false,
-			default_handler,
-			infos: [(handler1.0, handler1.1, options1), (get_invalid_no_modrm_handler().0, get_invalid_no_modrm_handler().1, 0)],
-			info_options: options1,
-		}
+		(
+			OpCodeHandler_Options1632::decode,
+			Self {
+				has_modrm: false,
+				default_handler,
+				infos: [(handler1.0, handler1.1, options1), (get_invalid_no_modrm_handler().0, get_invalid_no_modrm_handler().1, 0)],
+				info_options: options1,
+			},
+		)
 	}
 
+	#[inline]
 	pub(super) fn new2(
 		default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler1: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options1: u32,
 		handler2: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options2: u32,
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(default_handler.1));
 		debug_assert!(!is_null_instance_handler(handler1.1));
 		debug_assert!(!is_null_instance_handler(handler2.1));
-		Self {
-			decode: OpCodeHandler_Options1632::decode,
-			has_modrm: false,
-			default_handler,
-			infos: [(handler1.0, handler1.1, options1), (handler2.0, handler2.1, options2)],
-			info_options: options1 | options2,
-		}
+		(
+			OpCodeHandler_Options1632::decode,
+			Self {
+				has_modrm: false,
+				default_handler,
+				infos: [(handler1.0, handler1.1, options1), (handler2.0, handler2.1, options2)],
+				info_options: options1 | options2,
+			},
+		)
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -439,7 +523,6 @@ impl OpCodeHandler_Options1632 {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Options {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	infos: [(OpCodeHandlerDecodeFn, &'static OpCodeHandler, u32); 2],
 	info_options: u32,
@@ -447,34 +530,40 @@ pub(super) struct OpCodeHandler_Options {
 
 impl OpCodeHandler_Options {
 	#[allow(trivial_casts)]
+	#[inline]
 	pub(super) fn new(
 		default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler1: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options1: u32,
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(default_handler.1));
 		debug_assert!(!is_null_instance_handler(handler1.1));
-		Self {
-			decode: OpCodeHandler_Options::decode,
-			has_modrm: false,
-			default_handler,
-			infos: [(handler1.0, handler1.1, options1), (get_invalid_no_modrm_handler().0, get_invalid_no_modrm_handler().1, 0)],
-			info_options: options1,
-		}
+		(
+			OpCodeHandler_Options::decode,
+			Self {
+				has_modrm: false,
+				default_handler,
+				infos: [(handler1.0, handler1.1, options1), (get_invalid_no_modrm_handler().0, get_invalid_no_modrm_handler().1, 0)],
+				info_options: options1,
+			},
+		)
 	}
 
+	#[inline]
 	pub(super) fn new2(
 		default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), handler1: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options1: u32,
 		handler2: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), options2: u32,
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(default_handler.1));
 		debug_assert!(!is_null_instance_handler(handler1.1));
 		debug_assert!(!is_null_instance_handler(handler2.1));
-		Self {
-			decode: OpCodeHandler_Options::decode,
-			has_modrm: false,
-			default_handler,
-			infos: [(handler1.0, handler1.1, options1), (handler2.0, handler2.1, options2)],
-			info_options: options1 | options2,
-		}
+		(
+			OpCodeHandler_Options::decode,
+			Self {
+				has_modrm: false,
+				default_handler,
+				infos: [(handler1.0, handler1.1, options1), (handler2.0, handler2.1, options2)],
+				info_options: options1 | options2,
+			},
+		)
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
@@ -501,19 +590,19 @@ impl OpCodeHandler_Options {
 #[repr(C)]
 pub(super) struct OpCodeHandler_Options_DontReadModRM {
 	has_modrm: bool,
-	decode: OpCodeHandlerDecodeFn,
 	default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	opt_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler),
 	flags: u32,
 }
 
 impl OpCodeHandler_Options_DontReadModRM {
+	#[inline]
 	pub(super) fn new(
 		default_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), opt_handler: (OpCodeHandlerDecodeFn, &'static OpCodeHandler), flags: u32,
-	) -> Self {
+	) -> (OpCodeHandlerDecodeFn, Self) {
 		debug_assert!(!is_null_instance_handler(default_handler.1));
 		debug_assert!(!is_null_instance_handler(opt_handler.1));
-		Self { decode: OpCodeHandler_Options_DontReadModRM::decode, has_modrm: true, default_handler, opt_handler, flags }
+		(OpCodeHandler_Options_DontReadModRM::decode, Self { has_modrm: true, default_handler, opt_handler, flags })
 	}
 
 	fn decode(self_ptr: *const OpCodeHandler, decoder: &mut Decoder<'_>, instruction: &mut Instruction) {
