@@ -28,7 +28,7 @@ use crate::instruction_internal;
 use crate::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
-use core::{mem, u16, u32, u8};
+use core::mem;
 
 /// GNU assembler (AT&T) formatter
 ///
@@ -164,7 +164,7 @@ impl GasFormatter {
 				| (InstrOpInfoFlags::SIZE_OVERRIDE_MASK << InstrOpInfoFlags::ADDR_SIZE_SHIFT)
 				| InstrOpInfoFlags::BND_PREFIX;
 			if ((prefix_seg as u32)
-				| instruction_internal::internal_has_any_of_xacquire_xrelease_lock_rep_repne_prefix(instruction)
+				| instruction_internal::internal_has_any_of_lock_rep_repne_prefix(instruction)
 				| ((op_info.flags as u32) & PREFIX_FLAGS))
 				!= 0
 			{
@@ -379,6 +379,10 @@ impl GasFormatter {
 				| InstrOpKind::RdSae
 				| InstrOpKind::RuSae
 				| InstrOpKind::RzSae
+				| InstrOpKind::Rn
+				| InstrOpKind::Rd
+				| InstrOpKind::Ru
+				| InstrOpKind::Rz
 				| InstrOpKind::DeclareByte
 				| InstrOpKind::DeclareWord
 				| InstrOpKind::DeclareDword
@@ -390,7 +394,6 @@ impl GasFormatter {
 				| InstrOpKind::MemorySegDI
 				| InstrOpKind::MemorySegEDI
 				| InstrOpKind::MemorySegRDI
-				| InstrOpKind::Memory64
 				| InstrOpKind::Memory => return false,
 			}
 		}
@@ -425,6 +428,21 @@ impl GasFormatter {
 	fn format_operand(&mut self, instruction: &Instruction, output: &mut dyn FormatterOutput, op_info: &InstrOpInfo<'_>, operand: u32) {
 		debug_assert!(operand < op_info.op_count as u32);
 
+		#[cfg(feature = "mvex")]
+		let mvex_rm_operand = {
+			if IcedConstants::is_mvex(instruction.code()) {
+				let op_count = instruction.op_count();
+				debug_assert_ne!(op_count, 0);
+				if instruction.op_kind(op_count.wrapping_sub(1)) == OpKind::Immediate8 && op_info.op_count as u32 == op_count {
+					1
+				} else {
+					0
+				}
+			} else {
+				u32::MAX
+			}
+		};
+
 		let instruction_operand = op_info.instruction_index(operand);
 
 		if (op_info.flags & InstrOpInfoFlags::INDIRECT_OPERAND as u16) != 0 {
@@ -652,7 +670,7 @@ impl GasFormatter {
 				} else if op_kind == InstrOpKind::Immediate8_2nd {
 					imm8 = instruction.immediate8_2nd();
 				} else {
-					imm8 = instruction.try_get_declare_byte_value(operand as usize).unwrap_or_default();
+					imm8 = instruction.get_declare_byte_value(operand as usize);
 				}
 				let mut number_options = NumberFormattingOptions::with_immediate(&self.d.options);
 				operand_options = FormatterOperandOptions::default();
@@ -702,7 +720,7 @@ impl GasFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to16 {
 					imm16 = instruction.immediate8to16() as u16;
 				} else {
-					imm16 = instruction.try_get_declare_word_value(operand as usize).unwrap_or_default();
+					imm16 = instruction.get_declare_word_value(operand as usize);
 				}
 				let mut number_options = NumberFormattingOptions::with_immediate(&self.d.options);
 				operand_options = FormatterOperandOptions::default();
@@ -752,7 +770,7 @@ impl GasFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to32 {
 					imm32 = instruction.immediate8to32() as u32;
 				} else {
-					imm32 = instruction.try_get_declare_dword_value(operand as usize).unwrap_or_default();
+					imm32 = instruction.get_declare_dword_value(operand as usize);
 				}
 				let mut number_options = NumberFormattingOptions::with_immediate(&self.d.options);
 				operand_options = FormatterOperandOptions::default();
@@ -804,7 +822,7 @@ impl GasFormatter {
 				} else if op_kind == InstrOpKind::Immediate64 {
 					imm64 = instruction.immediate64();
 				} else {
-					imm64 = instruction.try_get_declare_qword_value(operand as usize).unwrap_or_default();
+					imm64 = instruction.get_declare_qword_value(operand as usize);
 				}
 				let mut number_options = NumberFormattingOptions::with_immediate(&self.d.options);
 				operand_options = FormatterOperandOptions::default();
@@ -931,7 +949,6 @@ impl GasFormatter {
 			InstrOpKind::MemoryESRDI => {
 				self.format_memory(output, instruction, operand, instruction_operand, Register::ES, Register::RDI, Register::None, 0, 0, 0, 8)
 			}
-			InstrOpKind::Memory64 => {}
 
 			InstrOpKind::Memory => {
 				let displ_size = instruction.memory_displ_size();
@@ -1002,6 +1019,42 @@ impl GasFormatter {
 				&self.d.str_.rz_sae,
 				DecoratorKind::RoundingControl,
 			),
+			InstrOpKind::Rn => GasFormatter::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.rn,
+				DecoratorKind::RoundingControl,
+			),
+			InstrOpKind::Rd => GasFormatter::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.rd,
+				DecoratorKind::RoundingControl,
+			),
+			InstrOpKind::Ru => GasFormatter::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.ru,
+				DecoratorKind::RoundingControl,
+			),
+			InstrOpKind::Rz => GasFormatter::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.rz,
+				DecoratorKind::RoundingControl,
+			),
 		}
 
 		if operand + 1 == op_info.op_count as u32 && instruction_internal::internal_has_op_mask_or_zeroing_masking(instruction) {
@@ -1022,6 +1075,20 @@ impl GasFormatter {
 				);
 			}
 		}
+		#[cfg(feature = "mvex")]
+		if mvex_rm_operand == operand {
+			let conv = instruction.mvex_reg_mem_conv();
+			if conv != MvexRegMemConv::None {
+				let mvex = crate::mvex::get_mvex_info(instruction.code());
+				if mvex.conv_fn != MvexConvFn::None {
+					let tbl = if mvex.is_conv_fn_32() { &self.d.vec_.mvex_reg_mem_consts_32 } else { &self.d.vec_.mvex_reg_mem_consts_64 };
+					let s = tbl[conv as usize];
+					if s.len() != 0 {
+						Self::format_decorator(&self.d.options, output, instruction, operand, instruction_operand, s, DecoratorKind::SwizzleMemConv);
+					}
+				}
+			}
+		}
 	}
 
 	fn format_decorator(
@@ -1243,6 +1310,18 @@ impl GasFormatter {
 		if !bcst_to.is_default() {
 			GasFormatter::format_decorator(&self.d.options, output, instruction, operand, instruction_operand, bcst_to, DecoratorKind::Broadcast);
 		}
+		#[cfg(feature = "mvex")]
+		if instruction.is_mvex_eviction_hint() {
+			Self::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.mvex.eh,
+				DecoratorKind::EvictionHint,
+			);
+		}
 	}
 }
 
