@@ -7,6 +7,7 @@ use crate::iced_constants::IcedConstants;
 use alloc::boxed::Box;
 use alloc::string::String;
 use alloc::vec::Vec;
+use core::convert::TryInto;
 use lazy_static::lazy_static;
 
 lazy_static! {
@@ -20,9 +21,7 @@ lazy_static! {
 			v.push(FormatterString::new(s.clone()));
 			s.clear();
 		}
-		let v = v.into_boxed_slice();
-		debug_assert_eq!(v.len(), IcedConstants::REGISTER_ENUM_COUNT);
-		// SAFETY: Size is verified above
-		unsafe { Box::from_raw(Box::into_raw(v) as *mut [_; IcedConstants::REGISTER_ENUM_COUNT]) }
+		#[allow(clippy::unwrap_used)]
+		v.into_boxed_slice().try_into().ok().unwrap()
 	};
 }
