@@ -27,8 +27,6 @@ use crate::instruction_internal;
 use crate::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
-use core::{u16, u32, u8};
-use static_assertions::const_assert_eq;
 
 /// Intel formatter (same as Intel XED)
 ///
@@ -154,7 +152,7 @@ impl IntelFormatter {
 				| InstrOpInfoFlags::JCC_NOT_TAKEN
 				| InstrOpInfoFlags::JCC_TAKEN;
 			if ((prefix_seg as u32)
-				| instruction_internal::internal_has_any_of_xacquire_xrelease_lock_rep_repne_prefix(instruction)
+				| instruction_internal::internal_has_any_of_lock_rep_repne_prefix(instruction)
 				| ((op_info.flags as u32) & PREFIX_FLAGS))
 				!= 0
 			{
@@ -361,7 +359,6 @@ impl IntelFormatter {
 				| InstrOpKind::MemorySegDI
 				| InstrOpKind::MemorySegEDI
 				| InstrOpKind::MemorySegRDI
-				| InstrOpKind::Memory64
 				| InstrOpKind::Memory => return false,
 			}
 		}
@@ -396,6 +393,21 @@ impl IntelFormatter {
 	fn format_operand(&mut self, instruction: &Instruction, output: &mut dyn FormatterOutput, op_info: &InstrOpInfo<'_>, operand: u32) {
 		debug_assert!(operand < op_info.op_count as u32);
 
+		#[cfg(feature = "mvex")]
+		let mvex_rm_operand = {
+			if IcedConstants::is_mvex(instruction.code()) {
+				let op_count = instruction.op_count();
+				debug_assert_ne!(op_count, 0);
+				if instruction.op_kind(op_count.wrapping_sub(1)) == OpKind::Immediate8 {
+					op_count.wrapping_sub(2)
+				} else {
+					op_count.wrapping_sub(1)
+				}
+			} else {
+				u32::MAX
+			}
+		};
+
 		let instruction_operand = op_info.instruction_index(operand);
 
 		let flow_control;
@@ -648,7 +660,7 @@ impl IntelFormatter {
 				} else if op_kind == InstrOpKind::Immediate8_2nd {
 					imm8 = instruction.immediate8_2nd();
 				} else {
-					imm8 = instruction.try_get_declare_byte_value(operand as usize).unwrap_or_default();
+					imm8 = instruction.get_declare_byte_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -699,7 +711,7 @@ impl IntelFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to16 {
 					imm16 = instruction.immediate8to16() as u16;
 				} else {
-					imm16 = instruction.try_get_declare_word_value(operand as usize).unwrap_or_default();
+					imm16 = instruction.get_declare_word_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -750,7 +762,7 @@ impl IntelFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to32 {
 					imm32 = instruction.immediate8to32() as u32;
 				} else {
-					imm32 = instruction.try_get_declare_dword_value(operand as usize).unwrap_or_default();
+					imm32 = instruction.get_declare_dword_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -803,7 +815,7 @@ impl IntelFormatter {
 				} else if op_kind == InstrOpKind::Immediate64 {
 					imm64 = instruction.immediate64();
 				} else {
-					imm64 = instruction.try_get_declare_qword_value(operand as usize).unwrap_or_default();
+					imm64 = instruction.get_declare_qword_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -973,7 +985,6 @@ impl IntelFormatter {
 				8,
 				op_info.flags as u32,
 			),
-			InstrOpKind::Memory64 => {}
 
 			InstrOpKind::Memory => {
 				let displ_size = instruction.memory_displ_size();
@@ -1022,18 +1033,22 @@ impl IntelFormatter {
 		if operand == 0 && instruction_internal::internal_has_rounding_control_or_sae(instruction) {
 			let rc = instruction.rounding_control();
 			if rc != RoundingControl::None && can_show_rounding_control(instruction, &self.d.options) {
-				const_assert_eq!(RoundingControl::None as u32, 0);
-				const_assert_eq!(RoundingControl::RoundToNearest as u32, 1);
-				const_assert_eq!(RoundingControl::RoundDown as u32, 2);
-				const_assert_eq!(RoundingControl::RoundUp as u32, 3);
-				const_assert_eq!(RoundingControl::RoundTowardZero as u32, 4);
+				let dec_str = if IcedConstants::is_mvex(instruction.code()) {
+					if instruction.suppress_all_exceptions() {
+						self.d.vec_.intel_rc_sae_strings[rc as usize]
+					} else {
+						self.d.vec_.intel_rc_strings[rc as usize]
+					}
+				} else {
+					self.d.vec_.intel_rc_sae_strings[rc as usize]
+				};
 				IntelFormatter::format_decorator(
 					&self.d.options,
 					output,
 					instruction,
 					operand,
 					instruction_operand,
-					self.d.vec_.intel_rc_strings[rc as usize - 1],
+					dec_str,
 					DecoratorKind::RoundingControl,
 				);
 			} else if instruction.suppress_all_exceptions() {
@@ -1048,6 +1063,20 @@ impl IntelFormatter {
 				);
 			}
 		}
+		#[cfg(feature = "mvex")]
+		if mvex_rm_operand == operand {
+			let conv = instruction.mvex_reg_mem_conv();
+			if conv != MvexRegMemConv::None {
+				let mvex = crate::mvex::get_mvex_info(instruction.code());
+				if mvex.conv_fn != MvexConvFn::None {
+					let tbl = if mvex.is_conv_fn_32() { &self.d.vec_.mvex_reg_mem_consts_32 } else { &self.d.vec_.mvex_reg_mem_consts_64 };
+					let s = tbl[conv as usize];
+					if s.len() != 0 {
+						Self::format_decorator(&self.d.options, output, instruction, operand, instruction_operand, s, DecoratorKind::SwizzleMemConv);
+					}
+				}
+			}
+		}
 	}
 
 	fn format_flow_control(d: &SelfData, output: &mut dyn FormatterOutput, flags: u32, operand_options: FormatterOperandOptions) {
@@ -1349,6 +1378,18 @@ impl IntelFormatter {
 		if !bcst_to.is_default() {
 			IntelFormatter::format_decorator(&self.d.options, output, instruction, operand, instruction_operand, bcst_to, DecoratorKind::Broadcast);
 		}
+		#[cfg(feature = "mvex")]
+		if instruction.is_mvex_eviction_hint() {
+			Self::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.mvex.eh,
+				DecoratorKind::EvictionHint,
+			);
+		}
 	}
 
 	#[allow(clippy::unwrap_used)]
