@@ -27,8 +27,6 @@ use crate::instruction_internal;
 use crate::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
-use core::{u16, u32, u8};
-use static_assertions::const_assert_eq;
 
 /// Masm formatter
 ///
@@ -148,7 +146,7 @@ impl MasmFormatter {
 		if (mnemonic_options & FormatMnemonicOptions::NO_PREFIXES) == 0 && (op_info.flags & InstrOpInfoFlags::MNEMONIC_IS_DIRECTIVE as u16) == 0 {
 			let prefix_seg = instruction.segment_prefix();
 			if ((prefix_seg as u32)
-				| instruction_internal::internal_has_any_of_xacquire_xrelease_lock_rep_repne_prefix(instruction)
+				| instruction_internal::internal_has_any_of_lock_rep_repne_prefix(instruction)
 				| ((op_info.flags as u32) & (InstrOpInfoFlags::JCC_NOT_TAKEN | InstrOpInfoFlags::JCC_TAKEN | InstrOpInfoFlags::BND_PREFIX)))
 				!= 0
 			{
@@ -335,7 +333,6 @@ impl MasmFormatter {
 				| InstrOpKind::MemorySegDI
 				| InstrOpKind::MemorySegEDI
 				| InstrOpKind::MemorySegRDI
-				| InstrOpKind::Memory64
 				| InstrOpKind::Memory => return false,
 			}
 		}
@@ -370,6 +367,21 @@ impl MasmFormatter {
 	fn format_operand(&mut self, instruction: &Instruction, output: &mut dyn FormatterOutput, op_info: &InstrOpInfo<'_>, operand: u32) {
 		debug_assert!(operand < op_info.op_count as u32);
 
+		#[cfg(feature = "mvex")]
+		let mvex_rm_operand = {
+			if IcedConstants::is_mvex(instruction.code()) {
+				let op_count = instruction.op_count();
+				debug_assert_ne!(op_count, 0);
+				if instruction.op_kind(op_count.wrapping_sub(1)) == OpKind::Immediate8 {
+					op_count.wrapping_sub(2)
+				} else {
+					op_count.wrapping_sub(1)
+				}
+			} else {
+				u32::MAX
+			}
+		};
+
 		let instruction_operand = op_info.instruction_index(operand);
 
 		let flow_control;
@@ -602,7 +614,7 @@ impl MasmFormatter {
 				} else if op_kind == InstrOpKind::Immediate8_2nd {
 					imm8 = instruction.immediate8_2nd();
 				} else {
-					imm8 = instruction.try_get_declare_byte_value(operand as usize).unwrap_or_default();
+					imm8 = instruction.get_declare_byte_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -657,7 +669,7 @@ impl MasmFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to16 {
 					imm16 = instruction.immediate8to16() as u16;
 				} else {
-					imm16 = instruction.try_get_declare_word_value(operand as usize).unwrap_or_default();
+					imm16 = instruction.get_declare_word_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -712,7 +724,7 @@ impl MasmFormatter {
 				} else if op_kind == InstrOpKind::Immediate8to32 {
 					imm32 = instruction.immediate8to32() as u32;
 				} else {
-					imm32 = instruction.try_get_declare_dword_value(operand as usize).unwrap_or_default();
+					imm32 = instruction.get_declare_dword_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -769,7 +781,7 @@ impl MasmFormatter {
 				} else if op_kind == InstrOpKind::Immediate64 {
 					imm64 = instruction.immediate64();
 				} else {
-					imm64 = instruction.try_get_declare_qword_value(operand as usize).unwrap_or_default();
+					imm64 = instruction.get_declare_qword_value(operand as usize);
 				}
 				operand_options = FormatterOperandOptions::default();
 				if let Some(ref symbol) = if let Some(ref mut symbol_resolver) = self.symbol_resolver {
@@ -943,7 +955,6 @@ impl MasmFormatter {
 				8,
 				op_info.flags as u32,
 			),
-			InstrOpKind::Memory64 => {}
 
 			InstrOpKind::Memory => {
 				let displ_size = instruction.memory_displ_size();
@@ -992,23 +1003,25 @@ impl MasmFormatter {
 		if operand + 1 == op_info.op_count as u32 && instruction_internal::internal_has_rounding_control_or_sae(instruction) {
 			let rc = instruction.rounding_control();
 			if rc != RoundingControl::None && can_show_rounding_control(instruction, &self.d.options) {
-				const_assert_eq!(RoundingControl::None as u32, 0);
-				const_assert_eq!(RoundingControl::RoundToNearest as u32, 1);
-				const_assert_eq!(RoundingControl::RoundDown as u32, 2);
-				const_assert_eq!(RoundingControl::RoundUp as u32, 3);
-				const_assert_eq!(RoundingControl::RoundTowardZero as u32, 4);
-				output.write(" ", FormatterTextKind::Text);
+				let dec_str = if IcedConstants::is_mvex(instruction.code()) {
+					if instruction.suppress_all_exceptions() {
+						self.d.vec_.masm_rc_sae_strings[rc as usize]
+					} else {
+						self.d.vec_.masm_rc_strings[rc as usize]
+					}
+				} else {
+					self.d.vec_.masm_rc_sae_strings[rc as usize]
+				};
 				MasmFormatter::format_decorator(
 					&self.d.options,
 					output,
 					instruction,
 					operand,
 					instruction_operand,
-					self.d.vec_.masm_rc_strings[rc as usize - 1],
+					dec_str,
 					DecoratorKind::RoundingControl,
 				);
 			} else if instruction.suppress_all_exceptions() {
-				output.write(" ", FormatterTextKind::Text);
 				MasmFormatter::format_decorator(
 					&self.d.options,
 					output,
@@ -1020,6 +1033,20 @@ impl MasmFormatter {
 				);
 			}
 		}
+		#[cfg(feature = "mvex")]
+		if mvex_rm_operand == operand {
+			let conv = instruction.mvex_reg_mem_conv();
+			if conv != MvexRegMemConv::None {
+				let mvex = crate::mvex::get_mvex_info(instruction.code());
+				if mvex.conv_fn != MvexConvFn::None {
+					let tbl = if mvex.is_conv_fn_32() { &self.d.vec_.mvex_reg_mem_consts_32 } else { &self.d.vec_.mvex_reg_mem_consts_64 };
+					let s = tbl[conv as usize];
+					if s.len() != 0 {
+						Self::format_decorator(&self.d.options, output, instruction, operand, instruction_operand, s, DecoratorKind::SwizzleMemConv);
+					}
+				}
+			}
+		}
 	}
 
 	fn format_decorator(
@@ -1248,6 +1275,18 @@ impl MasmFormatter {
 			}
 			output.write("]", FormatterTextKind::Punctuation);
 		}
+		#[cfg(feature = "mvex")]
+		if instruction.is_mvex_eviction_hint() {
+			Self::format_decorator(
+				&self.d.options,
+				output,
+				instruction,
+				operand,
+				instruction_operand,
+				&self.d.str_.mvex.eh,
+				DecoratorKind::EvictionHint,
+			);
+		}
 	}
 
 	#[allow(clippy::too_many_arguments)]
