@@ -8,6 +8,7 @@ use crate::Register;
 use alloc::boxed::Box;
 use alloc::string::String;
 use alloc::vec::Vec;
+use core::convert::TryInto;
 use core::fmt::Write;
 use lazy_static::lazy_static;
 
@@ -21,9 +22,7 @@ lazy_static! {
 			v[Register::ST0 as usize + i] = FormatterString::new(s.clone());
 			s.clear();
 		}
-		let v = v.into_boxed_slice();
-		debug_assert_eq!(v.len(), IcedConstants::REGISTER_ENUM_COUNT);
-		// SAFETY: Size is verified above
-		unsafe { Box::from_raw(Box::into_raw(v) as *mut [_; IcedConstants::REGISTER_ENUM_COUNT]) }
+		#[allow(clippy::unwrap_used)]
+		v.into_boxed_slice().try_into().ok().unwrap()
 	};
 }
