@@ -34,13 +34,7 @@ impl IcedError {
 }
 
 #[cfg(feature = "std")]
-impl Error for IcedError {
-	// Required since MSRV < 1.42.0
-	#[allow(clippy::missing_inline_in_public_items)]
-	fn description(&self) -> &str {
-		&self.error
-	}
-}
+impl Error for IcedError {}
 
 impl fmt::Display for IcedError {
 	#[inline]
