@@ -11,9 +11,6 @@ use crate::iced_constants::IcedConstants;
 #[cfg(feature = "encoder")]
 use crate::iced_error::IcedError;
 use crate::*;
-use core::u32;
-#[cfg(feature = "encoder")]
-use core::{i16, i32, i8, u16, u8};
 #[cfg(any(feature = "encoder", feature = "fast_fmt", feature = "instr_info"))]
 use static_assertions::const_assert_eq;
 
@@ -30,20 +27,6 @@ pub(crate) fn internal_set_code_size(this: &mut Instruction, new_value: CodeSize
 	this.flags1 |= (new_value as u32) << InstrFlags1::CODE_SIZE_SHIFT;
 }
 
-#[cfg(feature = "encoder")]
-#[must_use]
-#[inline]
-pub(crate) fn internal_has_repe_prefix_has_xrelease_prefix(this: &Instruction) -> bool {
-	(this.flags1 & (InstrFlags1::REPE_PREFIX | InstrFlags1::XRELEASE_PREFIX)) != 0
-}
-
-#[cfg(feature = "encoder")]
-#[must_use]
-#[inline]
-pub(crate) fn internal_has_repne_prefix_has_xacquire_prefix(this: &Instruction) -> bool {
-	(this.flags1 & (InstrFlags1::REPNE_PREFIX | InstrFlags1::XACQUIRE_PREFIX)) != 0
-}
-
 #[cfg(feature = "instr_info")]
 #[must_use]
 #[inline]
@@ -59,13 +42,8 @@ pub(crate) fn internal_set_has_repe_prefix(this: &mut Instruction) {
 
 #[cfg(any(feature = "gas", feature = "intel", feature = "masm", feature = "nasm", feature = "fast_fmt"))]
 #[inline]
-pub(crate) fn internal_has_any_of_xacquire_xrelease_lock_rep_repne_prefix(this: &Instruction) -> u32 {
-	this.flags1
-		& (InstrFlags1::XACQUIRE_PREFIX
-			| InstrFlags1::XRELEASE_PREFIX
-			| InstrFlags1::LOCK_PREFIX
-			| InstrFlags1::REPE_PREFIX
-			| InstrFlags1::REPNE_PREFIX)
+pub(crate) fn internal_has_any_of_lock_rep_repne_prefix(this: &Instruction) -> u32 {
+	this.flags1 & (InstrFlags1::LOCK_PREFIX | InstrFlags1::REPE_PREFIX | InstrFlags1::REPNE_PREFIX)
 }
 
 #[cfg(any(feature = "gas", feature = "intel", feature = "masm", feature = "nasm", feature = "fast_fmt"))]
@@ -122,7 +100,7 @@ pub(crate) fn internal_set_immediate8(this: &mut Instruction, new_value: u32) {
 #[cfg(any(feature = "decoder", feature = "encoder"))]
 #[inline]
 pub(crate) fn internal_set_immediate8_2nd(this: &mut Instruction, new_value: u32) {
-	this.mem_displ = new_value;
+	this.mem_displ = new_value as u64;
 }
 
 #[cfg(any(feature = "decoder", feature = "encoder"))]
@@ -140,25 +118,13 @@ pub(crate) fn internal_set_immediate64_lo(this: &mut Instruction, new_value: u32
 #[cfg(feature = "decoder")]
 #[inline]
 pub(crate) fn internal_set_immediate64_hi(this: &mut Instruction, new_value: u32) {
-	this.mem_displ = new_value;
-}
-
-#[cfg(feature = "decoder")]
-#[inline]
-pub(crate) fn internal_set_memory_displacement64_lo(this: &mut Instruction, new_value: u32) {
-	this.mem_displ = new_value;
-}
-
-#[cfg(feature = "decoder")]
-#[inline]
-pub(crate) fn internal_set_memory_displacement64_hi(this: &mut Instruction, new_value: u32) {
-	this.mem_displ_hi = new_value;
+	this.mem_displ = new_value as u64;
 }
 
 #[cfg(feature = "decoder")]
 #[inline]
 pub(crate) fn internal_set_near_branch16(this: &mut Instruction, new_value: u32) {
-	this.immediate = new_value;
+	this.mem_displ = new_value as u64;
 }
 
 #[cfg(feature = "decoder")]
@@ -170,7 +136,7 @@ pub(crate) fn internal_set_far_branch16(this: &mut Instruction, new_value: u32)
 #[cfg(feature = "decoder")]
 #[inline]
 pub(crate) fn internal_set_far_branch_selector(this: &mut Instruction, new_value: u32) {
-	this.mem_displ = new_value;
+	this.mem_displ = new_value as u64;
 }
 
 #[cfg(feature = "fast_fmt")]
@@ -192,7 +158,7 @@ pub(crate) fn internal_op_register(this: &Instruction, operand: u32) -> Register
 }
 
 #[cfg(feature = "encoder")]
-#[cfg(not(feature = "no_evex"))]
+#[cfg(any(not(feature = "no_evex"), feature = "mvex"))]
 #[must_use]
 #[inline]
 pub(crate) fn internal_op_mask(this: &Instruction) -> u32 {
@@ -200,7 +166,7 @@ pub(crate) fn internal_op_mask(this: &Instruction) -> u32 {
 }
 
 #[cfg(feature = "decoder")]
-#[cfg(not(feature = "no_evex"))]
+#[cfg(any(not(feature = "no_evex"), feature = "mvex"))]
 #[inline]
 pub(crate) fn internal_set_op_mask(this: &mut Instruction, new_value: u32) {
 	debug_assert!(new_value <= 7);
@@ -208,7 +174,7 @@ pub(crate) fn internal_set_op_mask(this: &mut Instruction, new_value: u32) {
 }
 
 #[cfg(feature = "decoder")]
-#[cfg(not(feature = "no_evex"))]
+#[cfg(any(not(feature = "no_evex"), feature = "mvex"))]
 #[inline]
 pub(crate) fn internal_set_rounding_control(this: &mut Instruction, new_value: u32) {
 	debug_assert!(new_value < IcedConstants::ROUNDING_CONTROL_ENUM_COUNT as u32);
@@ -228,6 +194,13 @@ pub(crate) fn internal_set_declare_data_len(this: &mut Instruction, new_value: u
 	this.flags1 |= (new_value - 1) << InstrFlags1::DATA_LENGTH_SHIFT;
 }
 
+#[cfg(all(feature = "decoder", feature = "mvex"))]
+#[inline]
+pub(crate) fn internal_set_mvex_reg_mem_conv(this: &mut Instruction, new_value: u32) {
+	debug_assert!(new_value < IcedConstants::MVEX_REG_MEM_CONV_ENUM_COUNT as u32);
+	this.immediate |= new_value << MvexInstrFlags::MVEX_REG_MEM_CONV_SHIFT;
+}
+
 // GENERATOR-BEGIN: RegToAddrSize
 // ⚠️This was generated by GENERATOR!🦹‍♂️
 #[rustfmt::skip]
